UNIT General;
  {$N+}  { Enable 8087 Mode }
  {$G+}  { Enable 286 Instructions }

INTERFACE

TYPE
    NewDateTime=record
         Year         : Word;
         Month        : Byte;
         Day          : Byte;
         Hour         : Byte;
         Minute       : Byte;
         Second       : Byte;
         Millisecond  : Word;
      End;
    TFindData=record                 { LFindFirst/Next }
         Name         : String;      { File Name }
         ShortName    : String[13];  { Alternate File Name }
         Size         : Comp;        { File Size }
         Attr         : Word;        { File Attributes - see PASCAL help }
         TimeModified : NewDateTime;
         TimeAccessed : NewDateTime;
         TimeCreated  : NewDateTime;
      End;

FUNCTION  Hex(I : LongInt; Size : Byte) : String;
FUNCTION  GetKey(ErrorReturn : Word) : Word;
FUNCTION  UpStr(S : String) : String;
FUNCTION  FileTimeToDOSTime (FileTime:comp; var DT:NewDateTime) : Byte;
FUNCTION  LFindFirst (FileName : String; ReqdAttrs : Byte; AllowAttrs : Byte; VAR FindData : TFindData) : Word;
FUNCTION  LFindNext (Handle:word; var FindData:TFindData) : Byte;
FUNCTION  LFindClose (Handle:word) : Byte;

CONST
     {File Attributes}
     NoAttribs  : Byte = $00;
     ReadOnly   : Byte = $01;
     Hidden     : Byte = $02;
     SysFile    : Byte = $04;
     VolumeID   : Byte = $08;
     Directory  : Byte = $10;
     Archive    : Byte = $20;
     AnyFile    : Byte = $3F;

IMPLEMENTATION

CONST
     CRTC_ADDR = $3D4;
     SEQU_ADDR = $3C4;

VAR
   OldKey : Word;

FUNCTION Hex(I : LongInt; Size : Byte) : String;
CONST
     HexChar : Array[0..15] Of Char =
       ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
VAR S : String;
Begin
     S := '';
     If Size = 0 Then
     Begin
          While (I Shr (Size*4)) > 0 Do
            Inc(Size);
     End;
     While Size > 0 Do
     Begin
          S := S+HexChar[(I SHR (4*(Size-1))) AND $F];
          Dec(Size);
     End;
     Hex := '$'+S;
End;

FUNCTION  GetKey(ErrorReturn : Word) : Word; Assembler;
ASM
  @001C:
   Mov  Bx, $0040            { BX = $0040 }
   Mov  Es, Bx               { ES = $0040 }
   Mov  Bx, $001C            { ES:BX = $0040:$001C }
   Mov  Cx, Es:[Bx]          { CX = Current Buffer Location }

   Mov  Bx, $001A            { ES:BX = $0040:$001A }
   Mov  Ax, Es:[Bx]          { AX = Buffer Read Marker }
   Cmp  Ax, Cx               { CX = Current Buffer Location }
   Je  @Clear                { If Buffers are in same location, exit }
   Inc  Ax; Inc  Ax          { CX = Buffer Read Marker + 2 }
   Cmp  Ax, $3C              { If Read Buffer = $3E Then set to $1E }
   Jle @Normal               { If Not last position, goto Normal }
   Mov  Ax, $1E              { If Last Position, correct addr }
   Mov  Es:[Bx], Ax          { Store Keystroke in Location at $001A }
   Mov  Ax, Es:[$3C]         { Get Keystroke in Location at [$001A] }
   Jmp @End                { RET would be faster }
  @Normal:
   Mov  Es:[Bx], Ax          { Store Keystroke in Location at $001A }
   Mov  Bx, Ax               { Get New Buffer Addr }
   Mov  Ax, Es:[Bx-$02]      { Get Keystroke in Location at [$001A] }
   Mov  OldKey, Ax           { New Key, so update X }
   Jmp @End                { RET would be faster }
  @Clear:
   Mov Ax, ErrorReturn       { GetKey = ErrorReturn }
  @End:
End;

FUNCTION  UpStr(S : String) : String;
VAR Temp : Byte;
Begin
     For Temp := 1 to Length(S) Do
         S[Temp] := UPCase(S[Temp]);
     UpStr := S;
End;

FUNCTION FileTimeToDOSTime (FileTime:comp; var DT:NewDateTime) : Byte; Assembler;
 ASM
    Push Ds
    Mov  Ax, $71A7
    Xor  Bx, Bx
    Push Ss
    Pop  Ds
    Lea  Si, FileTime
    Int  $21
    Pop  Ds
    Mov  Bx, Word Ptr DT
    Mov  Ax, Dx
    Shr  Dx, 9
    Mov  Word Ptr DS:[Bx], Dx
    Add  Word Ptr DS:[Bx], 1980
    Mov  Dx, Ax
    Shr  Dx, 5
    And  Dx, $0F
    Mov  Byte Ptr DS:[Bx+2], Dl
    And  Ax, $1F
    Mov  Byte Ptr DS:[Bx+3], Al
    Mov  Ax, Cx
    Shr  Cx, 11
    Mov  Byte Ptr DS:[Bx+4], Cl
    Mov  Cx, Ax
    Shr  Cx, 5
    And  Cx, $3F
    Mov  Byte Ptr DS:[Bx+5], Cl
    And  Ax, $1F
    Shl  Ax, 1
    Mov  Byte Ptr DS:[Bx+6], Cl
    Jc  @End
    Mov  Al, 0
   @End:
 END;

FUNCTION LFindFirst (FileName : String; ReqdAttrs : Byte; AllowAttrs : Byte; VAR FindData : TFindData) : Word;
 VAR
    P  : String;
    FD : Record
             FileAttributes : LongInt;
             CreationTime   : Comp;
             AccessTime     : Comp;
             WriteTime      : Comp;
             FileSizeHigh   : LongInt;
             FileSizeLow    : LongInt;
             Reserved       : Comp;
             FullName       : Array [0..259] of char;
             ShortName      : Array [0..13] of char
         End;
    I  : Byte;
    Flags : Word;

 Begin
      P := FileName+#0;
      ASM
         Stc
         Push Ds
         Mov  Ax, $714E
         Mov  Cl, AllowAttrs
         Mov  Ch, ReqdAttrs
         Lea  Dx, P[1]
         Xor  Si, Si
         Mov  Di, Ss
         Mov  Ds, Di
         Mov  Es, Di
         Lea  Di, Fd
         Int  $21
         Pop  Ds
         Mov  Flags, Ax
         Jnc @End
         Mov  Flags, 65535
        @End:
      END;
      Move(FD.FullName, FindData.Name[1], 254);
      For I := 1 to 255 Do If FindData.Name[I] = #0 Then
          Begin FindData.Name[0] := CHR(I-1); Break; End;
      Move(FD.ShortName, FindData.ShortName[1], 14);
      For I := 1 to 14 Do If FindData.ShortName[I] = #0 Then
          Begin
               If I = 1 Then FindData.ShortName := FindData.Name
                  ELSE FindData.ShortName[0] := CHR(I-1);
               Break;
          End;
      FindData.Size  := FD.FileSizeHigh SHL 16 + FD.FileSizeLow;
      FindData.Attr  := FD.FileAttributes;
      LFindFirst := Flags;
End;

FUNCTION LFindNext (Handle:word; var FindData:TFindData) : Byte;
 VAR
    FD : Record
             FileAttributes : LongInt;
             CreationTime   : Comp;
             AccessTime     : Comp;
             WriteTime      : Comp;
             FileSizeHigh   : LongInt;
             FileSizeLow    : LongInt;
             Reserved       : Comp;
             FullName       : Array [0..259] of char;
             ShortName      : Array [0..13] of char
         End;
    I : Byte;
    Flags : Word;

 Begin
      ASM
         Stc
         Mov  Ax, $714F
         Mov  Bx, Handle
         Xor  Si, Si
         Push Ss
         Pop  Es
         Lea  Di, FD
         Int  $21
         Mov  Flags, 0
         Jnc @End
         Mov  Flags, Ax
        @End:
      END;
      Move (FD.FullName, FindData.Name[1], 255);
      For I := 1 to 255 Do If FindData.Name[I] = #0 Then
          Begin FindData.Name[0] := CHR(I-1); Break; End;
      Move(FD.ShortName, FindData.ShortName[1], 14);
      For I := 1 to 14 Do If FindData.ShortName[I] = #0 Then
          Begin
               If I = 1 Then FindData.ShortName := FindData.Name
                  ELSE FindData.ShortName[0] := CHR(I-1);
               Break;
          End;
      FindData.Size  := FD.FileSizeHigh SHL 16 + FD.FileSizeLow;
      FindData.Attr  := FD.FileAttributes;
      FileTimeToDOSTime(FD.CreationTime, FindData.TimeCreated);
      FileTimeToDOSTime(FD.AccessTime, FindData.TimeAccessed);
      FileTimeToDOSTime(FD.WriteTime, FindData.TimeModified);
      LFindNext := Flags;
end;

FUNCTION LFindClose (Handle:word) : Byte; Assembler;
 ASM
    Stc
    Mov  Ax, $71A1
    Mov  Bx, Handle
    Int  $21
    Jc  @End
    Xor  Al, Al
   @End:
 END;



End.